@@ -7,7 +7,7 @@ SporeStack API request/response models
 
 from typing import Dict, List, Optional
 
-from pydantic import BaseModel
+from pydantic import BaseModel, Field
 
 from .models import Flavor, NetworkInterface, Payment
 
@@ -36,6 +36,25 @@ class TokenBalance:
         usd: str
 
 
+class ServerQuote:
+    url = "/server/quote"
+    method = "GET"
+
+    """Takes days and flavor as parameters."""
+
+    class Response(BaseModel):
+        cents: int = Field(
+            default=..., ge=1, title="Cents", description="(US) cents", example=1_000_00
+        )
+        usd: str = Field(
+            default=...,
+            min_length=5,
+            title="USD",
+            description="USD in $1,000.00 format",
+            example="$1,000.00",
+        )
+
+
 class ServerLaunch:
     url = "/server/{machine_id}/launch"
     method = "POST"
