@@ -7,10 +7,14 @@ import logging
 import os
 import time
 from pathlib import Path
-from typing import Any, Dict, Optional
+from typing import TYPE_CHECKING, Any, Dict, Optional
 
 import typer
 
+if TYPE_CHECKING:
+    from . import api
+
+
 HELP = """
 SporeStack Python CLI
 
@@ -19,7 +23,7 @@ SPORESTACK_ENDPOINT
 *or*
 SPORESTACK_USE_TOR_ENDPOINT
 
-TOR_PROXY (defaults to socks5h://127.0.0.1:9050 which is fine for most)
+TOR_PROXY (defaults to socks5://127.0.0.1:9050 which is fine for most)
 """
 
 _home = os.getenv("HOME", None)
@@ -82,14 +86,19 @@ Press ctrl+c to abort."""
 @server_cli.command()
 def launch(
     hostname: str = "",
-    days: int = typer.Option(...),
-    operating_system: str = typer.Option(...),
+    days: int = typer.Option(
+        ..., min=1, max=90, help="Number of days the server should run for."
+    ),
+    operating_system: str = typer.Option(..., help="Example: debian-11"),
     ssh_key_file: Path = DEFAULT_SSH_KEY_FILE,
     flavor: str = DEFAULT_FLAVOR,
     token: str = DEFAULT_TOKEN,
     region: Optional[str] = None,
     quote: bool = typer.Option(True, help="Require manual price confirmation."),
-    autorenew: bool = typer.Option(False, help="BETA: Automatically renew server."),
+    autorenew: bool = typer.Option(False, help="Automatically renew server."),
+    wait: bool = typer.Option(
+        True, help="Wait for server to be assigned an IP address."
+    ),
 ) -> None:
     """
     Launch a server on SporeStack.
@@ -99,8 +108,10 @@ def launch(
 
     from . import utils
     from .api_client import APIClient
+    from .client import Client
 
     api_client = APIClient(api_endpoint=get_api_endpoint())
+    client = Client(api_client=api_client, client_token=_token)
 
     typer.echo(f"Loading SSH key from {ssh_key_file}...")
     if not ssh_key_file.exists():
@@ -114,20 +125,9 @@ def launch(
     machine_id = utils.random_machine_id()
 
     if quote:
-        response = api_client.server_launch(
-            machine_id=machine_id,
-            days=days,
-            flavor=flavor,
-            operating_system=operating_system,
-            ssh_key=ssh_key,
-            region=region,
-            token=_token,
-            quote=True,
-            hostname=hostname,
-            autorenew=autorenew,
-        )
+        quote_response = client.server_quote(days=days, flavor=flavor)
 
-        msg = f"Is {response.payment.usd} for {days} day(s) of {flavor} okay?"
+        msg = f"Is {quote_response.usd} for {days} day(s) of {flavor} okay?"
         typer.echo(msg, err=True)
         input("[Press ctrl+c to cancel, or enter to accept.]")
 
@@ -141,34 +141,37 @@ def launch(
             err=True,
         )
 
-    tries = 360
-    while tries > 0:
-        response = api_client.server_launch(
-            machine_id=machine_id,
-            days=days,
-            flavor=flavor,
-            operating_system=operating_system,
-            ssh_key=ssh_key,
-            region=region,
-            token=_token,
-            hostname=hostname,
-            autorenew=autorenew,
-        )
-        if response.created is True:
-            break
-        typer.echo("Waiting for server to build...", err=True)
-        tries = tries + 1
-        # Waiting for server to spin up.
-        time.sleep(10)
-
-    if response.created is False:
-        typer.echo("Server creation failed, tries exceeded.", err=True)
-        raise typer.Exit(code=1)
-
-    typer.echo(
-        pretty_machine_info(api_client.server_info(machine_id=machine_id).dict())
+    server = client.token.launch_server(
+        machine_id=machine_id,
+        days=days,
+        flavor=flavor,
+        operating_system=operating_system,
+        ssh_key=ssh_key,
+        region=region,
+        hostname=hostname,
+        autorenew=autorenew,
     )
 
+    if wait:
+        tries = 360
+        while tries > 0:
+            response = server.info()
+            if response.ipv4 != "":
+                break
+            typer.echo("Waiting for server to build...", err=True)
+            tries = tries + 1
+            # Waiting for server to spin up.
+            time.sleep(10)
+
+        if response.ipv4 == "":
+            typer.echo("Server creation failed, tries exceeded.", err=True)
+            raise typer.Exit(code=1)
+        else:
+            print_machine_info(response)
+            return
+
+    print_machine_info(server.info())
+
 
 @server_cli.command()
 def topup(
@@ -271,6 +274,38 @@ def pretty_machine_info(info: Dict[str, Any]) -> str:
     return msg
 
 
+def print_machine_info(info: "api.ServerInfo.Response") -> None:
+    if info.hostname != "":
+        typer.echo(f"Hostname: {info.hostname}")
+    else:
+        typer.echo("Hostname: (none) (No hostname set)")
+
+    typer.echo(f"Machine ID (keep this secret!): {info.machine_id}")
+    if info.ipv6 != "":
+        typer.echo(f"IPv6: {info.ipv6}")
+    else:
+        typer.echo("IPv6: (Not yet assigned)")
+    if info.ipv4 != "":
+        typer.echo(f"IPv4: {info.ipv4}")
+    else:
+        typer.echo("IPv4: (Not yet assigned)")
+    typer.echo(f"Region: {info.region}")
+    typer.echo(f"Flavor: {info.flavor.slug}")
+    human_expiration = time.strftime(
+        "%Y-%m-%d %H:%M:%S %z", time.localtime(info.expiration)
+    )
+    typer.echo(f"Expiration: {info.expiration} ({human_expiration})")
+    typer.echo(f"Token: {info.token}")
+    if info.deleted:
+        typer.echo("Server was deleted!")
+    else:
+        typer.echo(f"Running: {info.running}")
+        time_to_live = info.expiration - int(time.time())
+        hours = time_to_live // 3600
+        typer.echo(f"Server will be deleted in {hours} hours.")
+        typer.echo(f"Autorenew: {info.autorenew}")
+
+
 @server_cli.command(name="list")
 def server_list(
     token: str = DEFAULT_TOKEN,
@@ -307,27 +342,9 @@ def server_list(
         if hostname == "":
             if info.machine_id in machine_id_hostnames:
                 hostname = machine_id_hostnames[info.machine_id]
-        if hostname != "":
-            typer.echo(f"Hostname: {hostname}")
+        info.hostname = hostname
 
-        typer.echo(f"Machine ID (keep this secret!): {info.machine_id}")
-        typer.echo(f"IPv6: {info.network_interfaces[0].ipv6}")
-        typer.echo(f"IPv4: {info.network_interfaces[0].ipv4}")
-        typer.echo(f"Region: {info.region}")
-        typer.echo(f"Flavor: {info.flavor.slug}")
-        human_expiration = time.strftime(
-            "%Y-%m-%d %H:%M:%S %z", time.localtime(info.expiration)
-        )
-        typer.echo(f"Expiration: {info.expiration} ({human_expiration})")
-        typer.echo(f"Token: {info.token}")
-        if info.deleted:
-            typer.echo("Server was deleted!")
-        else:
-            typer.echo(f"Running: {info.running}")
-            time_to_live = info.expiration - int(time.time())
-            hours = time_to_live // 3600
-            typer.echo(f"Server will be deleted in {hours} hours.")
-            typer.echo(f"Autorenew: {info.autorenew}")
+        print_machine_info(info)
 
         printed_machine_ids.append(info.machine_id)
 
@@ -406,9 +423,7 @@ def info(hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN) -
     from .api_client import APIClient
 
     api_client = APIClient(api_endpoint=get_api_endpoint())
-    typer.echo(
-        pretty_machine_info(api_client.server_info(machine_id=machine_id).dict())
-    )
+    print_machine_info(api_client.server_info(machine_id=machine_id))
 
 
 @server_cli.command(name="json")
@@ -759,7 +774,7 @@ def version() -> None:
 def api_endpoint() -> None:
     """
     Prints the selected API endpoint: Env var: SPORESTACK_ENDPOINT,
-    or, SPORESTACK_USE_TOR=1
+    or, SPORESTACK_USE_TOR_ENDPOINT=1
     """
     from . import api_client
 
